@@ -135,21 +135,25 @@ void cpgbbuf(void)
 
 /***************************************************************
  * cpgbeg -- open a graphics device
- * Status: PARTIALLY IMPLEMENTED
+ * Status: IMPLEMENTED
  ***************************************************************/
 int cpgbeg(int unit, const char *file, int nxsub, int nysub)
 {
-  return cpgopen(file);
+  int id = cpgopen(file);
+  if (id > 0 && (nxsub > 1 || nysub > 1)) {
+     cpgsubp(nxsub, nysub);
+  }
+  return id;
 }
 
 /***************************************************************
  * cpgbin -- histogram of binned data
- * Status: NOT IMPLEMENTED
+ * Status: IMPLEMENTED
  ***************************************************************/
 void cpgbin(int nbin, const float *x, const float *data, \
  Logical center)
 {
-
+  giza_histogram_binned_float(nbin,x,data,(int) center);
 }
 
 /***************************************************************
@@ -387,12 +391,12 @@ void cpghi2D(const float *data, int nxv, int nyv, int ix1, \
 
 /***************************************************************
  * cpghist -- histogram of unbinned data
- * Status: NOT IMPLEMENTED
+ * Status: IMPLEMENTED
  ***************************************************************/
 void cpghist(int n, const float *data, float datmin, float datmax, \
  int nbin, int pgflag)
 {
-
+  giza_histogram_float(n,data,datmin,datmax,nbin,pgflag);
 }
 
 /***************************************************************
@@ -530,11 +534,11 @@ void cpgpage(void)
 
 /***************************************************************
  * cpgpanl -- switch to a different panel on the view surface
- * Status: NOT IMPLEMENTED
+ * Status: IMPLEMENTED
  ***************************************************************/
 void cpgpanl(int nxc, int nyc)
 {
-
+  giza_set_panel(nxc, nyc);
 }
 
 /***************************************************************
@@ -1052,11 +1056,11 @@ void cpgstbg(int tbci)
 
 /***************************************************************
  * cpgsubp -- subdivide view surface into panels
- * Status: NOT IMPLEMENTED
+ * Status: IMPLEMENTED
  ***************************************************************/
 void cpgsubp(int nxsub, int nysub)
 {
-
+  giza_subpanel(nxsub, nysub);
 }
 
 /***************************************************************
