@@ -44,7 +44,7 @@
  *  -j1            :- The inclusive range of data to render in the y direction
  *  -j2            :- The inclusive range of data to render in the y direction
  *  -valMin        :- The value in data that gets assign the colour corresponding to zero
- *                    on the colour ramp (The ramp is set by giza_set_colour_ramp
+ *                    on the colour ramp (The ramp is set by giza_set_colour_table)
  *  -valMax        :- The value in data that gets assigned the colour corresponding to one
  *                    on the colour ramp
  *  -extend        :- Option for how to deal with image at edges
@@ -55,6 +55,8 @@
  *  -1 or GIZA_EXTEND_REPEAT  :- periodic tiling
  *  -2 or GIZA_EXTEND_REFLECT :- reflective boundary
  *  -3 or GIZA_EXTEND_PAD     :- pad by extending last few pixels
+ *
+ * See Also: giza_set_colour_table
  */
 void
 giza_render (int sizex, int sizey, const double* data, int i1, int i2,
@@ -128,7 +130,7 @@ _giza_render (int sizex, int sizey, const double* data, int i1, int i2,
   _giza_set_trans (GIZA_TRANS_WORLD);
   cairo_matrix_init (&mat, affine[0], affine[1], affine[2], affine[3],
 		     affine[4], affine[5]);
-  cairo_transform (context, &mat);
+  cairo_transform (Dev[id].context, &mat);
 
   /* allocate data for the pixmap */
 #if CAIRO_VERSION >= CAIRO_VERSION_ENCODE(1, 6, 0)
@@ -183,9 +185,9 @@ _giza_render (int sizex, int sizey, const double* data, int i1, int i2,
 						width, height, stride);
 
   /* paint the pixmap to the primary surface */
-  cairo_set_source_surface (context, pixmap, 0, 0);
-  cairo_pattern_set_extend (cairo_get_source (context), cairoextendtype);
-  cairo_paint (context);
+  cairo_set_source_surface (Dev[id].context, pixmap, 0, 0);
+  cairo_pattern_set_extend (cairo_get_source (Dev[id].context), cairoextendtype);
+  cairo_paint (Dev[id].context);
 
   /* clean up and restore settings */
   _giza_set_trans (oldTrans);
@@ -193,10 +195,7 @@ _giza_render (int sizex, int sizey, const double* data, int i1, int i2,
   cairo_surface_destroy (pixmap);
   free (pixdata);
 
-  if (!Sets.buf)
-    {
-      giza_flush_device ();
-    }
+  giza_flush_device ();
 }
 
 /**
@@ -289,7 +288,7 @@ _giza_render_float (int sizex, int sizey, const float* data, int i1,
   cairo_matrix_init (&mat, (double) affine[0], (double) affine[1],
 		     (double) affine[2], (double) affine[3],
 		     (double) affine[4], (double) affine[5]);
-  cairo_transform (context, &mat);
+  cairo_transform (Dev[id].context, &mat);
 
 #if CAIRO_VERSION >= CAIRO_VERSION_ENCODE(1, 6, 0)
   stride = cairo_format_stride_for_width (format, width);
@@ -340,18 +339,15 @@ _giza_render_float (int sizex, int sizey, const float* data, int i1,
   pixmap = cairo_image_surface_create_for_data (pixdata, format,
 						width, height, stride);
 
-  cairo_set_source_surface (context, pixmap, 0, 0);
-  cairo_pattern_set_extend (cairo_get_source (context), cairoextendtype);
-  cairo_paint (context);
+  cairo_set_source_surface (Dev[id].context, pixmap, 0, 0);
+  cairo_pattern_set_extend (cairo_get_source (Dev[id].context), cairoextendtype);
+  cairo_paint (Dev[id].context);
 
   _giza_set_trans (oldTrans);
   giza_set_colour_index (oldCi);
   cairo_surface_destroy (pixmap);
   free (pixdata);
-  if (!Sets.buf)
-    {
-      giza_flush_device ();
-    }
+  giza_flush_device ();
 }
 
 /**
@@ -525,7 +521,7 @@ giza_draw_pixels (int sizex, int sizey, const int* idata, int i1, int i2,
   double dypix = (ymax - ymin)/((double) height);
   cairo_matrix_init (&mat, dxpix, 0.,0., dypix,
 		     xmin, ymin);
-  cairo_transform (context, &mat);
+  cairo_transform (Dev[id].context, &mat);
 
   /* allocate data for the pixmap */
 #if CAIRO_VERSION >= CAIRO_VERSION_ENCODE(1, 6, 0)
@@ -553,9 +549,9 @@ giza_draw_pixels (int sizex, int sizey, const int* idata, int i1, int i2,
 						width, height, stride);
 
   /* paint the pixmap to the primary surface */
-  cairo_set_source_surface (context, pixmap, 0, 0);
-  cairo_pattern_set_extend (cairo_get_source (context), cairoextendtype);
-  cairo_paint (context);
+  cairo_set_source_surface (Dev[id].context, pixmap, 0, 0);
+  cairo_pattern_set_extend (cairo_get_source (Dev[id].context), cairoextendtype);
+  cairo_paint (Dev[id].context);
 
   /* clean up and restore settings */
   _giza_set_trans (oldTrans);
@@ -563,10 +559,7 @@ giza_draw_pixels (int sizex, int sizey, const int* idata, int i1, int i2,
   cairo_surface_destroy (pixmap);
   free (pixdata);
 
-  if (!Sets.buf)
-    {
-      giza_flush_device ();
-    }
+  giza_flush_device ();
 }
 
 /**
