@@ -1,7 +1,7 @@
 /* giza - a scientific plotting library built on cairo
  *
  * Copyright (c) 2010      James Wetter and Daniel Price
- * Copyright (c) 2010-2012 Daniel Price
+ * Copyright (c) 2010-2014 Daniel Price
  *
  * This library is free software; and you are welcome to redistribute
  * it under the terms of the GNU General Public License
@@ -38,10 +38,10 @@
  * device that can be drawn to.
  *
  * Input:
- *  -xleft    :- The x coordinate of the left edge of the view port
- *  -xright   :- The x coordinate of the right edge of the view port
- *  -ybottom  :- The y coordinate of the bottom edge of the view port
- *  -ytop     :- The y coordinate of the top edge of the view port
+ *  -xleft    :- The x coordinate of the left edge of the viewport
+ *  -xright   :- The x coordinate of the right edge of the viewport
+ *  -ybottom  :- The y coordinate of the bottom edge of the viewport
+ *  -ytop     :- The y coordinate of the top edge of the viewport
  */
 void
 giza_set_viewport (double xleft, double xright, double ybottom, double ytop)
@@ -57,6 +57,11 @@ giza_set_viewport (double xleft, double xright, double ybottom, double ytop)
 
       _giza_get_default_viewport(&Dev[id].VP.xmin,&Dev[id].VP.xmax,&Dev[id].VP.ymin,&Dev[id].VP.ymax);
 
+      xmin = Dev[id].VP.xmin;
+      xmax = Dev[id].VP.xmax;
+      ymin = Dev[id].VP.ymin;
+      ymax = Dev[id].VP.ymax;
+
     } else {
 
       if (xleft > xright)
@@ -86,7 +91,13 @@ giza_set_viewport (double xleft, double xright, double ybottom, double ytop)
 
   _giza_set_trans (GIZA_TRANS_NORM);
   cairo_reset_clip (Dev[id].context);
-  cairo_rectangle (Dev[id].context, xmin, ymin, xmax - xmin, ymax - ymin);
+  
+  double x0 = (Dev[id].ix - 1)*Dev[id].panelwidth/Dev[id].width;
+  double y0 = (Dev[id].ny - Dev[id].iy)*Dev[id].panelheight/Dev[id].height;
+  double xfac = Dev[id].panelwidth/Dev[id].width;
+  double yfac = Dev[id].panelheight/Dev[id].height;
+  
+  cairo_rectangle (Dev[id].context, xmin*xfac + x0, ymin*yfac + y0, (xmax - xmin)*xfac, (ymax - ymin)*yfac);
 
   int clip;
   giza_get_clipping(&clip);
@@ -217,7 +228,12 @@ giza_get_viewport_float (int units, float *x1, float *x2, float *y1, float *y2)
 }
 
 /**
- * Sets the viewport to the default
+ * Settings: giza_set_viewport_default
+ *
+ * Synopsis: Sets the viewport to the default settings
+ *
+ * See Also: giza_set_viewport
+ * 
  */
 void
 giza_set_viewport_default (void)
@@ -237,15 +253,20 @@ _giza_get_default_viewport (double *xmin, double *xmax, double *ymin, double *ym
 {
   double xch, ych;
   giza_get_character_size(GIZA_UNITS_NORMALIZED, &xch, &ych);
-  *xmin = 4.*xch;
-  *xmax = *xmin + (1. - 8.*xch);
+  *xmin = 4.*ych;
+  *xmax = *xmin + (1. - 8.*ych);
   *ymin = 4.*ych;
   *ymax = *ymin + (1. - 8.*ych);
 
 }
 
 /**
- * Sets the viewport in inches (this is for PGPLOT-compatibility)
+ * Settings: giza_set_viewport_inches
+ *
+ * Synopsis: Sets the viewport size in inches
+ *
+ * See Also: giza_set_viewport
+ * 
  */
 void
 giza_set_viewport_inches (double xleftin, double xrightin, double ybottomin, double ytopin)
@@ -265,6 +286,14 @@ giza_set_viewport_inches (double xleftin, double xrightin, double ybottomin, dou
 
 }
 
+/**
+ * Settings: giza_set_viewport_inches_float
+ *
+ * Synopsis: Same as giza_set_viewport_inches but takes floats
+ *
+ * See Also: giza_set_viewport_inches
+ * 
+ */
 void
 giza_set_viewport_inches_float (float xleftin, float xrightin, float ybottomin, float ytopin)
 {
