@@ -5,5 +5,5 @@
 #
 # DJP, Feb 2014
 #
-opts='-r 10 -b:v 2M -bt 5M -vcodec mpeg4 -vf setpts=4.*PTS'
+opts='-r 10 -vb 50M -bt 100M -vcodec mpeg4 -vf setpts=4.*PTS'
 ffmpeg -i splash_%04d.png $opts movie.mp4
