@@ -15,7 +15,7 @@
 !  a) You must cause the modified files to carry prominent notices
 !     stating that you changed the files and the date of any change.
 !
-!  Copyright (C) 2005-2013 Daniel Price. All rights reserved.
+!  Copyright (C) 2005-2014 Daniel Price. All rights reserved.
 !  Contact: daniel.price@monash.edu
 !
 !-----------------------------------------------------------------
@@ -32,7 +32,8 @@
 !---------------------------------------------------------------------------
 module asciiutils
  implicit none
- public :: read_asciifile,get_ncolumns,ncolumnsline,safename,basename,cstring
+ public :: read_asciifile,get_ncolumns,get_nrows,ncolumnsline,safename,basename
+ public :: cstring,fstring
  public :: string_replace, string_delete, nheaderlines, string_sub
  public :: ucase,lcase
  public :: get_line_containing
@@ -305,6 +306,28 @@ subroutine get_ncolumns(lunit,ncolumns,nheaderlines)
 
 end subroutine get_ncolumns
 
+!---------------------------------------------------------------------------
+! utility to work out number of rows in file
+!---------------------------------------------------------------------------
+subroutine get_nrows(lunit,nheaderlines,nlines)
+ implicit none
+ integer, intent(in)  :: lunit,nheaderlines
+ integer, intent(out) :: nlines
+ integer :: ierr,i
+
+ rewind(lunit)
+ ierr = 0
+ do i=1,nheaderlines
+    read(lunit,*,iostat=ierr)
+ enddo
+ nlines = 0
+ do while (ierr==0)
+    read(lunit,*,iostat=ierr)
+    if (ierr==0) nlines = nlines + 1
+ enddo
+
+end subroutine get_nrows
+
 !---------------------------------------------------------------------------
 !
 ! function returning the number of columns of real numbers from a given line
@@ -446,6 +469,27 @@ function cstring(string)
 
 end function cstring
 
+!---------------------------------------------------------------------------
+!
+! function to safely convert a string from c format (ie. with a terminating
+! ascii null character) back to a normal Fortran string
+!
+!---------------------------------------------------------------------------
+function fstring(array)
+ use, intrinsic :: iso_c_binding, only:c_char
+ implicit none
+ character(kind=c_char), dimension(:), intent(in) :: array
+ character(len=size(array)-1) :: fstring
+ integer :: i
+
+ fstring = ''
+ do i=1,size(array)
+    if (array(i).eq.achar(0)) exit
+    fstring(i:i) = array(i)
+ enddo
+
+end function fstring
+
 !---------------------------------------------------------------------------
 !
 ! subroutine to replace a matching section of a string with another
