@@ -15,7 +15,7 @@
 !  a) You must cause the modified files to carry prominent notices
 !     stating that you changed the files and the date of any change.
 !
-!  Copyright (C) 2005-2014 Daniel Price. All rights reserved.
+!  Copyright (C) 2005-2015 Daniel Price. All rights reserved.
 !  Contact: daniel.price@monash.edu
 !
 !-----------------------------------------------------------------
@@ -89,6 +89,9 @@ module exact
   integer, parameter :: iexact_rochelobe = 15
   !--C-shock
   real :: machs,macha
+  !--gamma, for manual setting
+  real :: gamma_exact
+  logical :: use_gamma_exact
   !
   !--sort these into a namelist for input/output
   !
@@ -104,7 +107,7 @@ module exact
        iprofile,Msphere,rsoft,icolpoten,icolfgrav,Mstar,Rtorus,distortion, &
        Mring,Rring,viscnu,nfunc,funcstring,cs,Kdrag,rhozero,rdust_to_gas, &
        mprim,msec,ixcolfile,iycolfile,xshock,totmass,machs,macha,&
-       use_sink_data,xprim,xsec,nfiles
+       use_sink_data,xprim,xsec,nfiles,gamma_exact,use_gamma_exact
 
   public :: defaults_set_exact,submenu_exact,options_exact,read_exactparams
   public :: exact_solution
@@ -184,6 +187,9 @@ contains
 !   C-shock
     machs = 50. ! sonic Mach number
     macha = 5.  ! Alfvenic Mach number
+!   gamma, if not read from file
+    gamma_exact = 5./3.
+    use_gamma_exact = .false.
 
 !   arbitrary function
     nfunc = 1
@@ -215,7 +221,7 @@ contains
     use asciiutils,    only:get_ncolumns,get_nrows,string_replace
     integer, intent(inout) :: iexact
     integer :: ierr,itry,i,ncols,nheaderlines,nadjust,nrows
-    logical :: ians,iexist,ltmp
+    logical :: ians,iexist,ltmp,prompt_for_gamma
     character(len=len(filename_exact)) :: filename_tmp
     character(len=4) :: str
 
@@ -242,6 +248,7 @@ contains
     !
     !--enter parameters for various exact solutions
     !
+    prompt_for_gamma = .false.
     select case(iexact)
     case(1)
        call prompt('enter number of functions to plot ',nfunc,1,maxexact)
@@ -368,13 +375,16 @@ contains
              call prompt('enter velocity to right of shock ',v_R)
              call prompt('enter dust-to-gas ratio ',rdust_to_gas,0.)             
           endif
-       endif      
+       endif
+       prompt_for_gamma = .true.
     case(4)
        call prompt('enter density of ambient medium ',rhosedov,0.0)
        call prompt('enter blast wave energy E ',esedov,0.0)
+       prompt_for_gamma = .true.
     case(5)
        call prompt('enter polytropic k ',polyk)
        call prompt('enter total mass ',totmass)
+       prompt_for_gamma = .true.
     case(6)
        print "(a)",' toy star: '
        call read_exactparams(iexact,trim(rootname(1)),ierr)
@@ -410,6 +420,7 @@ contains
              call prompt('enter vyx amplitude ',ctstar2)
           endif
        endif
+       prompt_for_gamma = .true.
     !case(7)
     !   call prompt('enter y-plot to place sine wave on',iwaveploty,1)
     !   call prompt('enter x-plot to place sine wave on',iwaveplotx,1)
@@ -449,6 +460,7 @@ contains
        call prompt('enter distortion parameter ',distortion,1.,2.)
        if (abs(polyk-1.0).lt.tiny(polyk)) polyk = 0.0764
        call prompt('enter K in P= K*rho^gamma',polyk,0.)
+       prompt_for_gamma = .true.
     case(12)
        call prompt('enter mass of ring',Mring,0.)
        call prompt('enter radius of ring centre R0',Rring,0.)
@@ -475,6 +487,13 @@ contains
        call prompt('enter sonic Mach number',machs,0.)
        call prompt('enter Alfvenic Mach number ',macha,0.)
     end select
+    
+    if (prompt_for_gamma) then
+       call prompt('set adiabatic gamma manually? (no=read from dumps)',use_gamma_exact)
+       if (use_gamma_exact) then
+          call prompt('enter gamma',gamma_exact)
+       endif
+    endif
 
     return
   end subroutine submenu_exact
@@ -526,8 +545,11 @@ contains
     character(len=*), intent(in)  :: rootname
     integer,          intent(out) :: ierr
 
-    integer :: idash,nf,i,j,idrag,idum,linenum
+    integer :: idash,nf,i,j,idrag,idum,linenum,k,ieq,ierrs(6)
     character(len=len_trim(rootname)+8) :: filename
+    character(len=120) :: line
+    character(len=30)  :: var
+    logical            :: iexist
 
     idash = index(rootname,'_')
     if (idash.eq.0) idash = len_trim(rootname)+1
@@ -577,23 +599,59 @@ contains
        !--shock tube parameters from .shk file
        !
        filename = trim(rootname(1:idash-1))//'.shk'
-       open(UNIT=19,ERR=7701,FILE=filename,STATUS='old')
-       read(19,*,ERR=7777) rho_L, rho_R
-       read(19,*,ERR=7777) pr_L, pr_R
-       read(19,*,ERR=7777) v_L, v_R
-       close(UNIT=19)
-       print*,'>> read ',filename
-       print*,' rhoL, rho_R = ',rho_L,rho_R
-       print*,' pr_L, pr_R  = ',pr_L, pr_R
-       print*,' v_L,  v_R   = ',v_L, v_R
-       return
-7701   print*,'no file ',filename
-       ierr = 1
-       return
-7777   print*,'error reading ',filename
-       close(UNIT=19)
-       ierr = 2
-       return
+       inquire(file=filename,exist=iexist)
+       if (iexist) then
+          open(unit=19,file=filename,status='old',iostat=ierr)
+          if (ierr==0) then
+             read(19,*,iostat=ierrs(1)) rho_L, rho_R
+             read(19,*,iostat=ierrs(2)) pr_L, pr_R
+             read(19,*,iostat=ierrs(3)) v_L, v_R
+             if (any(ierrs(1:3)/=0)) then
+                print*,'error reading ',filename
+                ierr = 1
+             endif
+          endif
+          close(unit=19)
+       else
+          print*,'no file ',filename
+          !
+          ! look for .setup file for Phantom
+          !
+          filename= trim(rootname(1:idash-1))//'.setup'
+          inquire(file=filename,exist=iexist)
+          open(unit=19,file=filename,status='old',iostat=ierr)
+          ierrs(:) = 0
+          do while(ierr==0)
+             read(19,"(a)",iostat=ierr) line
+             ieq = index(line,'=')
+             if (ierr==0 .and. ieq > 1) then
+                var = trim(adjustl(line(1:ieq-1)))
+                select case(trim(var))
+                case('densleft')
+                   read(line(ieq+1:),*,iostat=ierrs(1)) rho_L
+                case('densright')
+                   read(line(ieq+1:),*,iostat=ierrs(1)) rho_R
+                case('prleft')
+                   read(line(ieq+1:),*,iostat=ierrs(1)) pr_L
+                case('prright')
+                   read(line(ieq+1:),*,iostat=ierrs(1)) pr_R
+                case('vxleft')
+                   read(line(ieq+1:),*,iostat=ierrs(1)) v_L
+                case('vxright')
+                   read(line(ieq+1:),*,iostat=ierrs(1)) v_R
+                end select
+             endif
+          enddo
+          ierr = 0
+          if (.not.iexist .or. any(ierrs(1:6)/=0)) ierr = 1
+       endif
+       if (iexist) then
+          print*,'>> read ',filename
+          print*,' rhoL, rho_R = ',rho_L,rho_R
+          print*,' pr_L, pr_R  = ',pr_L, pr_R
+          print*,' v_L,  v_R   = ',v_L, v_R
+       endif
+
 
     case(6)
        !
@@ -669,9 +727,16 @@ contains
           do i=1,linenum-1
              read(19,*,iostat=ierr)
           enddo
+          read(19,"(a)",iostat=ierr) line
           if (ierr.eq.0) then
-             read(19,*,iostat=ierr) idrag, idum, idum, Kdrag
-             print*,'>> read Kdrag = ',Kdrag,' from '//trim(filename)
+             k = index(line,'Kdrag =')
+             if (k > 0) then
+                read(line(k+7:),*,iostat=ierr) Kdrag
+                print*,'>> read Kdrag = ',Kdrag,' from '//trim(filename)
+             elseif (ierr.eq.0) then
+                read(line,*,iostat=ierr) idrag, idum, idum, Kdrag
+                print*,'>> read Kdrag = ',Kdrag,' from '//trim(filename)
+             endif
           else
              print*,'>> error reading Kdrag from '//trim(filename)
           endif
@@ -712,8 +777,10 @@ contains
   !-----------------------------------------------------------------------
 
   subroutine exact_solution(iexact,iplotx,iploty,itransx,itransy,igeom, &
-                            ndim,ndimV,time,xmin,xmax,gamma,xplot,yplot, &
+                            ndim,ndimV,time,xmin,xmax,gamma,xplot,yplot,&
+                            itag,iamtype,noftype,iplot_type, &
                             pmassmin,pmassmax,npart,imarker,unitsx,unitsy,irescale,iaxisy)
+    use params,          only:int1,maxparttypes
     use labels,          only:ix,irad,iBfirst,ivx,irho,ike,iutherm,ih,ipr,iJfirst,&
                               irhorestframe,is_coord,ideltav,idustfrac
     use filenames,       only:ifileopen,rootname
@@ -744,12 +811,16 @@ contains
     real,    intent(in) :: time,xmin,xmax,gamma,unitsx,unitsy
     real,    intent(in) :: pmassmin,pmassmax
     real,    intent(in) :: xplot(npart),yplot(npart)
-    logical, intent(in) :: irescale
+    integer, intent(in) :: itag(npart)
+    integer(int1), intent(in) :: iamtype(:)
+    integer,       intent(in) :: noftype(maxparttypes)
+    logical,       intent(in) :: iplot_type(maxparttypes)
+    logical,       intent(in) :: irescale
 
     real, parameter :: zero = 1.e-10
     integer :: i,ierr,iexactpts,iCurrentColour,iCurrentLineStyle,LineStyle
     real, allocatable :: xexact(:),yexact(:),xtemp(:)
-    real :: dx,timei
+    real :: dx,timei,gammai
     character(len=len(filename_exact)) :: filename_tmp
 
     !
@@ -809,6 +880,13 @@ contains
     else
        timei = 0.
     endif
+    if (use_gamma_exact .and. gamma_exact >= 1.) then
+       gammai = gamma_exact
+    elseif (gamma >= 1.) then
+       gammai = gamma
+    else
+       gammai = 5./3.
+    endif
 
     select case(iexact)
     case(1) ! arbitrary function parsing
@@ -816,8 +894,8 @@ contains
           if ((iplotx.eq.iexactplotx(i) .and. iploty.eq.iexactploty(i)) .or. iexactploty(i).eq.0) then
              call exact_function(funcstring(i),xexact,yexact,timei,ierr)
              !--plot each solution separately and calculate errors
-             call plot_exact_solution(itransx,itransy,iexactpts,npart,&
-                                      xexact,yexact,xplot,yplot,imarker,iaxisy)
+             call plot_exact_solution(itransx,itransy,iexactpts,npart,xexact,yexact,xplot,yplot, &
+                                      itag,iamtype,noftype,iplot_type,xmin,xmax,imarker,iaxisy)
              ierr = 1 ! indicate that we have already plotted the solution
           endif
        enddo
@@ -841,8 +919,8 @@ contains
                 !--change line style between files
                 LineStyle = mod(iExactLineStyle+i-1,plotlib_maxlinestyle)
                 !--plot each solution separately and calculate errors
-                call plot_exact_solution(itransx,itransy,iexactpts,npart,&
-                                         xexact,yexact,xplot,yplot,imarker,iaxisy,ls=LineStyle)
+                call plot_exact_solution(itransx,itransy,iexactpts,npart,xexact,yexact,xplot,yplot,&
+                                         itag,iamtype,noftype,iplot_type,xmin,xmax,imarker,iaxisy,ls=LineStyle)
                 ierr = 1 ! indicate that we have already plotted the solution
              endif
           endif
@@ -850,22 +928,22 @@ contains
     case(3)! shock tube
        if (iplotx.eq.ix(1) .and. igeom.le.1) then
           if (iploty.eq.irho) then
-             call exact_shock(1,timei,gamma,rho_L,rho_R,pr_L,pr_R,v_L,v_R, &
+             call exact_shock(1,timei,gammai,rho_L,rho_R,pr_L,pr_R,v_L,v_R, &
                               rdust_to_gas,xexact,yexact,ierr)
           elseif (iploty.eq.ipr) then
-             call exact_shock(2,timei,gamma,rho_L,rho_R,pr_L,pr_R,v_L,v_R, &
+             call exact_shock(2,timei,gammai,rho_L,rho_R,pr_L,pr_R,v_L,v_R, &
                               rdust_to_gas,xexact,yexact,ierr)   
           elseif (iploty.eq.ivx) then
-              call exact_shock(3,timei,gamma,rho_L,rho_R,pr_L,pr_R,v_L,v_R, &
+              call exact_shock(3,timei,gammai,rho_L,rho_R,pr_L,pr_R,v_L,v_R, &
                               rdust_to_gas,xexact,yexact,ierr)
           elseif (iploty.eq.iutherm) then
-              call exact_shock(4,timei,gamma,rho_L,rho_R,pr_L,pr_R,v_L,v_R, &
+              call exact_shock(4,timei,gammai,rho_L,rho_R,pr_L,pr_R,v_L,v_R, &
                               rdust_to_gas,xexact,yexact,ierr)
           elseif (iploty.eq.ideltav) then
-              call exact_shock(5,timei,gamma,rho_L,rho_R,pr_L,pr_R,v_L,v_R, &
+              call exact_shock(5,timei,gammai,rho_L,rho_R,pr_L,pr_R,v_L,v_R, &
                               rdust_to_gas,xexact,yexact,ierr)
           elseif (iploty.eq.idustfrac) then
-              call exact_shock(6,timei,gamma,rho_L,rho_R,pr_L,pr_R,v_L,v_R, &
+              call exact_shock(6,timei,gammai,rho_L,rho_R,pr_L,pr_R,v_L,v_R, &
                               rdust_to_gas,xexact,yexact,ierr)
           endif
        endif
@@ -874,23 +952,23 @@ contains
        ! this subroutine does change xexact
        if (iplotx.eq.irad .or. (igeom.eq.3 .and. iplotx.eq.ix(1))) then
           if (iploty.eq.irho) then
-             call exact_sedov(1,timei,gamma,rhosedov,esedov,xmax,xexact,yexact,ierr)
+             call exact_sedov(1,timei,gammai,rhosedov,esedov,xmax,xexact,yexact,ierr)
           elseif (iploty.eq.ipr) then
-             call exact_sedov(2,timei,gamma,rhosedov,esedov,xmax,xexact,yexact,ierr)
+             call exact_sedov(2,timei,gammai,rhosedov,esedov,xmax,xexact,yexact,ierr)
           elseif (iploty.eq.iutherm) then
-             call exact_sedov(3,timei,gamma,rhosedov,esedov,xmax,xexact,yexact,ierr)
+             call exact_sedov(3,timei,gammai,rhosedov,esedov,xmax,xexact,yexact,ierr)
           elseif (iploty.eq.ike) then
-             call exact_sedov(4,timei,gamma,rhosedov,esedov,xmax,xexact,yexact,ierr)
+             call exact_sedov(4,timei,gammai,rhosedov,esedov,xmax,xexact,yexact,ierr)
           elseif (iploty.eq.ivx .and. igeom.eq.3) then
-             call exact_sedov(5,timei,gamma,rhosedov,esedov,xmax,xexact,yexact,ierr)
+             call exact_sedov(5,timei,gammai,rhosedov,esedov,xmax,xexact,yexact,ierr)
           endif
        elseif (igeom.le.1 .and. is_coord(iplotx,ndim) .and. is_coord(iploty,ndim)) then
-          call exact_sedov(0,timei,gamma,rhosedov,esedov,xmax,xexact,yexact,ierr)
+          call exact_sedov(0,timei,gammai,rhosedov,esedov,xmax,xexact,yexact,ierr)
        endif
 
     case(5)! polytrope
        if (iploty.eq.irho .and. (iplotx.eq.irad .or.(igeom.eq.3 .and. iplotx.eq.ix(1)))) then
-          call exact_polytrope(gamma,polyk,totmass,xexact,yexact,iexactpts,ierr)
+          call exact_polytrope(gammai,polyk,totmass,xexact,yexact,iexactpts,ierr)
        endif
 
     case(6)! toy star
@@ -905,30 +983,30 @@ contains
           !
           if (iplotx.eq.ix(1) .or. iplotx.eq.irad) then! if x axis is x or r
              if (iploty.eq.irho) then
-                call exact_toystar1D(1,timei,gamma,htstar,atstar,ctstar,sigma,norder, &
+                call exact_toystar1D(1,timei,gammai,htstar,atstar,ctstar,sigma,norder, &
                                    xexact,yexact,iexactpts,ierr)
              elseif (iploty.eq.ipr) then
-                call exact_toystar1D(2,timei,gamma,htstar,atstar,ctstar,sigma,norder, &
+                call exact_toystar1D(2,timei,gammai,htstar,atstar,ctstar,sigma,norder, &
                                    xexact,yexact,iexactpts,ierr)
              elseif (iploty.eq.iutherm) then
-                call exact_toystar1D(3,timei,gamma,htstar,atstar,ctstar,sigma,norder, &
+                call exact_toystar1D(3,timei,gammai,htstar,atstar,ctstar,sigma,norder, &
                                    xexact,yexact,iexactpts,ierr)
              elseif (iploty.eq.ivx) then
-                call exact_toystar1D(4,timei,gamma,htstar,atstar,ctstar,sigma,norder, &
+                call exact_toystar1D(4,timei,gammai,htstar,atstar,ctstar,sigma,norder, &
                                    xexact,yexact,iexactpts,ierr)
              elseif (iploty.eq.iBfirst+1) then
-                call exact_toystar1D(5,timei,gamma,htstar,atstar,ctstar,sigma,norder, &
+                call exact_toystar1D(5,timei,gammai,htstar,atstar,ctstar,sigma,norder, &
                                    xexact,yexact,iexactpts,ierr)
              endif
           elseif (iplotx.eq.irho) then
              if (iploty.eq.iBfirst+1) then
-                call exact_toystar1D(6,timei,gamma,htstar,atstar,ctstar,sigma,norder, &
+                call exact_toystar1D(6,timei,gammai,htstar,atstar,ctstar,sigma,norder, &
                                    xexact,yexact,iexactpts,ierr)
              endif
           endif
 
           if (iploty.eq.iacplane) then! plot point on a-c plane
-             call exact_toystar1D(7,timei,gamma,htstar,atstar,ctstar,sigma,norder, &
+             call exact_toystar1D(7,timei,gammai,htstar,atstar,ctstar,sigma,norder, &
                                 xexact,yexact,iexactpts,ierr)
           endif
        else
@@ -938,34 +1016,34 @@ contains
           !
           if (igeom.eq.1 .and.((iplotx.eq.ix(1) .and. iploty.eq.ivx) &
                .or. (iplotx.eq.ix(2) .and. iploty.eq.ivx+1))) then
-             call exact_toystar2D(4,timei,gamma,polyk,totmass, &
+             call exact_toystar2D(4,timei,gammai,polyk,totmass, &
                   atstar,htstar,ctstar,norder,morder, &
                   alphatstar,betatstar,ctstar1,ctstar2,xexact,yexact,ierr)
           endif
           if (iplotx.eq.irad .or. (igeom.eq.2 .and. iplotx.eq.ix(1))) then
              if (iploty.eq.irho) then
-                call exact_toystar2D(1,timei,gamma,polyk,totmass, &
+                call exact_toystar2D(1,timei,gammai,polyk,totmass, &
                      atstar,htstar,ctstar,norder,morder, &
                      alphatstar,betatstar,ctstar1,ctstar2,xexact,yexact,ierr)
              elseif (iploty.eq.ipr) then
-                call exact_toystar2D(2,timei,gamma,polyk,totmass, &
+                call exact_toystar2D(2,timei,gammai,polyk,totmass, &
                      atstar,htstar,ctstar,norder,morder, &
                      alphatstar,betatstar,ctstar1,ctstar2,xexact,yexact,ierr)
              elseif (iploty.eq.iutherm) then
-                call exact_toystar2D(3,timei,gamma,polyk,totmass, &
+                call exact_toystar2D(3,timei,gammai,polyk,totmass, &
                      atstar,htstar,ctstar,norder,morder, &
                      alphatstar,betatstar,ctstar1,ctstar2,xexact,yexact,ierr)
              elseif (igeom.eq.2 .and. iploty.eq.ivx) then
-                call exact_toystar2D(4,timei,gamma,polyk,totmass, &
+                call exact_toystar2D(4,timei,gammai,polyk,totmass, &
                      atstar,htstar,ctstar,norder,morder, &
                      alphatstar,betatstar,ctstar1,ctstar2,xexact,yexact,ierr)
              elseif (iploty.eq.ike) then
-                call exact_toystar2D(5,timei,gamma,polyk,totmass, &
+                call exact_toystar2D(5,timei,gammai,polyk,totmass, &
                      atstar,htstar,ctstar,norder,morder, &
                      alphatstar,betatstar,ctstar1,ctstar2,xexact,yexact,ierr)
              endif
           elseif (is_coord(iplotx,ndim) .and. is_coord(iploty,ndim) .and. igeom.eq.1) then
-             call exact_toystar2D(0,timei,gamma,polyk,totmass, &
+             call exact_toystar2D(0,timei,gammai,polyk,totmass, &
                   atstar,htstar,ctstar,norder,morder, &
                   alphatstar,betatstar,ctstar1,ctstar2,xexact,yexact,ierr)
           endif
@@ -987,31 +1065,31 @@ contains
        ! this subroutine modifies xexact
        if (iplotx.eq.ix(1) .and. igeom.le.1) then
           if (iploty.eq.irho) then
-             call exact_mhdshock(1,ishk,timei,gamma,xmin,xmax,xshock, &
+             call exact_mhdshock(1,ishk,timei,gammai,xmin,xmax,xshock, &
                                  xexact,yexact,iexactpts,ierr)
           elseif (iploty.eq.ipr) then
-             call exact_mhdshock(2,ishk,timei,gamma,xmin,xmax,xshock, &
+             call exact_mhdshock(2,ishk,timei,gammai,xmin,xmax,xshock, &
                                  xexact,yexact,iexactpts,ierr)
           elseif (iploty.eq.ivx) then
-             call exact_mhdshock(3,ishk,timei,gamma,xmin,xmax,xshock, &
+             call exact_mhdshock(3,ishk,timei,gammai,xmin,xmax,xshock, &
                                  xexact,yexact,iexactpts,ierr)
           elseif (iploty.eq.ivx+1 .and. ndimV.gt.1) then
-             call exact_mhdshock(4,ishk,timei,gamma,xmin,xmax,xshock, &
+             call exact_mhdshock(4,ishk,timei,gammai,xmin,xmax,xshock, &
                                  xexact,yexact,iexactpts,ierr)
           elseif (iploty.eq.ivx+ndimV-1 .and. ndimV.gt.2) then
-             call exact_mhdshock(5,ishk,timei,gamma,xmin,xmax,xshock, &
+             call exact_mhdshock(5,ishk,timei,gammai,xmin,xmax,xshock, &
                                  xexact,yexact,iexactpts,ierr)
           elseif (iploty.eq.iBfirst+1 .and. ndimV.gt.1) then
-             call exact_mhdshock(6,ishk,timei,gamma,xmin,xmax,xshock, &
+             call exact_mhdshock(6,ishk,timei,gammai,xmin,xmax,xshock, &
                                  xexact,yexact,iexactpts,ierr)
           elseif (iploty.eq.iBfirst+ndimV-1 .and. ndimV.gt.2) then
-             call exact_mhdshock(7,ishk,timei,gamma,xmin,xmax,xshock, &
+             call exact_mhdshock(7,ishk,timei,gammai,xmin,xmax,xshock, &
                                  xexact,yexact,iexactpts,ierr)
           elseif (iploty.eq.iutherm) then
-             call exact_mhdshock(8,ishk,timei,gamma,xmin,xmax,xshock, &
+             call exact_mhdshock(8,ishk,timei,gammai,xmin,xmax,xshock, &
                                  xexact,yexact,iexactpts,ierr)
           elseif (iploty.eq.iBfirst) then
-             call exact_mhdshock(9,ishk,timei,gamma,xmin,xmax,xshock, &
+             call exact_mhdshock(9,ishk,timei,gammai,xmin,xmax,xshock, &
                                  xexact,yexact,iexactpts,ierr)
           endif
        endif
@@ -1055,28 +1133,28 @@ contains
     case(11) ! torus
        if (iplotx.eq.irad .or.(igeom.eq.3 .and. iplotx.eq.ix(1))) then
           if (iploty.eq.irho) then
-             call exact_torus(1,1,Mstar,Rtorus,polyk,distortion,gamma,xexact,yexact,ierr)
+             call exact_torus(1,1,Mstar,Rtorus,polyk,distortion,gammai,xexact,yexact,ierr)
           elseif (iploty.eq.ipr) then
-             call exact_torus(2,1,Mstar,Rtorus,polyk,distortion,gamma,xexact,yexact,ierr)
+             call exact_torus(2,1,Mstar,Rtorus,polyk,distortion,gammai,xexact,yexact,ierr)
           elseif (iploty.eq.iutherm) then
-             call exact_torus(3,1,Mstar,Rtorus,polyk,distortion,gamma,xexact,yexact,ierr)
+             call exact_torus(3,1,Mstar,Rtorus,polyk,distortion,gammai,xexact,yexact,ierr)
           endif
        !--pr vs z at r=Rtorus
        elseif (igeom.eq.2 .and. iplotx.eq.ix(3) .and.iploty.eq.ipr) then
-          call exact_torus(4,1,Mstar,Rtorus,polyk,distortion,gamma,xexact,yexact,ierr)
+          call exact_torus(4,1,Mstar,Rtorus,polyk,distortion,gammai,xexact,yexact,ierr)
        endif
        !--solutions for tokamak torus
        if (igeom.eq.4 .and. iplotx.eq.ix(1)) then
           if (iploty.eq.irho) then
-             call exact_torus(1,2,Mstar,Rtorus,polyk,distortion,gamma,xexact,yexact,ierr)
+             call exact_torus(1,2,Mstar,Rtorus,polyk,distortion,gammai,xexact,yexact,ierr)
           elseif (iploty.eq.ipr) then
-             call exact_torus(2,2,Mstar,Rtorus,polyk,distortion,gamma,xexact,yexact,ierr)
+             call exact_torus(2,2,Mstar,Rtorus,polyk,distortion,gammai,xexact,yexact,ierr)
           elseif (iploty.eq.iutherm) then
-             call exact_torus(3,2,Mstar,Rtorus,polyk,distortion,gamma,xexact,yexact,ierr)
+             call exact_torus(3,2,Mstar,Rtorus,polyk,distortion,gammai,xexact,yexact,ierr)
           elseif (iploty.eq.iBfirst+1 .and. iBfirst.gt.0) then
-             call exact_torus(4,2,Mstar,Rtorus,polyk,distortion,gamma,xexact,yexact,ierr)
+             call exact_torus(4,2,Mstar,Rtorus,polyk,distortion,gammai,xexact,yexact,ierr)
           elseif (iploty.eq.iJfirst+2 .and. iJfirst.gt.0) then
-             call exact_torus(5,2,Mstar,Rtorus,polyk,distortion,gamma,xexact,yexact,ierr)
+             call exact_torus(5,2,Mstar,Rtorus,polyk,distortion,gammai,xexact,yexact,ierr)
           endif
        endif
     case(12)
@@ -1088,15 +1166,15 @@ contains
     case(13) ! special relativistic shock tube
        if (iplotx.eq.ix(1) .and. igeom.le.1) then
           if (iploty.eq.irhorestframe) then
-             call exact_shock_sr(1,timei,gamma,rho_L,rho_R,pr_L,pr_R,v_L,v_R,xexact,yexact,ierr)
+             call exact_shock_sr(1,timei,gammai,rho_L,rho_R,pr_L,pr_R,v_L,v_R,xexact,yexact,ierr)
           elseif (iploty.eq.ipr) then
-             call exact_shock_sr(2,timei,gamma,rho_L,rho_R,pr_L,pr_R,v_L,v_R,xexact,yexact,ierr)
+             call exact_shock_sr(2,timei,gammai,rho_L,rho_R,pr_L,pr_R,v_L,v_R,xexact,yexact,ierr)
           elseif (iploty.eq.ivx) then
-             call exact_shock_sr(3,timei,gamma,rho_L,rho_R,pr_L,pr_R,v_L,v_R,xexact,yexact,ierr)
+             call exact_shock_sr(3,timei,gammai,rho_L,rho_R,pr_L,pr_R,v_L,v_R,xexact,yexact,ierr)
           elseif (iploty.eq.iutherm) then
-             call exact_shock_sr(4,timei,gamma,rho_L,rho_R,pr_L,pr_R,v_L,v_R,xexact,yexact,ierr)
+             call exact_shock_sr(4,timei,gammai,rho_L,rho_R,pr_L,pr_R,v_L,v_R,xexact,yexact,ierr)
           elseif (iploty.eq.irho) then
-             call exact_shock_sr(5,timei,gamma,rho_L,rho_R,pr_L,pr_R,v_L,v_R,xexact,yexact,ierr)
+             call exact_shock_sr(5,timei,gammai,rho_L,rho_R,pr_L,pr_R,v_L,v_R,xexact,yexact,ierr)
           endif
        endif
     case(14) ! dusty wave exact solution
@@ -1104,21 +1182,25 @@ contains
           if (iploty.eq.ivx) then
              !--plot gas solution and calculate errors
              call exact_dustywave(1,timei,ampl,cs,Kdrag,lambda,xzero,rhozero,rhozero*rdust_to_gas,xexact,yexact,ierr)
-             call plot_exact_solution(itransx,itransy,iexactpts,npart,xexact,yexact,xplot,yplot,imarker,iaxisy,ls=1)
+             call plot_exact_solution(itransx,itransy,iexactpts,npart,xexact,yexact,xplot,yplot, &
+                                      itag,iamtype,noftype,iplot_type,xmin,xmax,imarker,iaxisy,ls=1)
              !--plot dust solution
              if (Kdrag > 0.) then
                 call exact_dustywave(2,timei,ampl,cs,Kdrag,lambda,xzero,rhozero,rhozero*rdust_to_gas,xexact,yexact,ierr)
-                call plot_exact_solution(itransx,itransy,iexactpts,npart,xexact,yexact,xplot,yplot,imarker,iaxisy,ls=2,err=.false.)
+                call plot_exact_solution(itransx,itransy,iexactpts,npart,xexact,yexact,xplot,yplot, &
+                                         itag,iamtype,noftype,iplot_type,xmin,xmax,imarker,iaxisy,ls=2,err=.false.)
              endif
              ierr = 1
           elseif (iploty.eq.irho) then
              !--plot gas solution and calculate errors
              call exact_dustywave(3,timei,ampl,cs,Kdrag,lambda,xzero,rhozero,rhozero*rdust_to_gas,xexact,yexact,ierr)
-             call plot_exact_solution(itransx,itransy,iexactpts,npart,xexact,yexact,xplot,yplot,imarker,iaxisy,ls=1)
+             call plot_exact_solution(itransx,itransy,iexactpts,npart,xexact,yexact,xplot,yplot,&
+                                      itag,iamtype,noftype,iplot_type,xmin,xmax,imarker,iaxisy,ls=1)
              !--plot dust solution
              if (Kdrag > 0.) then
                 call exact_dustywave(4,timei,ampl,cs,Kdrag,lambda,xzero,rhozero,rhozero*rdust_to_gas,xexact,yexact,ierr)
-                call plot_exact_solution(itransx,itransy,iexactpts,npart,xexact,yexact,xplot,yplot,imarker,iaxisy,ls=2,err=.false.)
+                call plot_exact_solution(itransx,itransy,iexactpts,npart,xexact,yexact,xplot,yplot,&
+                                         itag,iamtype,noftype,iplot_type,xmin,xmax,imarker,iaxisy,ls=2,err=.false.)
              endif
              ierr = 1
           endif
@@ -1130,15 +1212,15 @@ contains
     case(16) ! C-shock
        if (ndim.ge.1 .and. iplotx.eq.ix(1) .and. igeom.le.1) then
           if (iploty.eq.irho) then
-             call exact_Cshock(1,timei,gamma,machs,macha,xmin,xmax,xexact,yexact,ierr)
+             call exact_Cshock(1,timei,gammai,machs,macha,xmin,xmax,xexact,yexact,ierr)
           elseif (iploty.eq.iBfirst+1 .and. iBfirst.gt.0) then
-             call exact_Cshock(2,timei,gamma,machs,macha,xmin,xmax,xexact,yexact,ierr)
+             call exact_Cshock(2,timei,gammai,machs,macha,xmin,xmax,xexact,yexact,ierr)
           elseif (iploty.eq.ivx) then
-             call exact_Cshock(3,timei,gamma,machs,macha,xmin,xmax,xexact,yexact,ierr)
+             call exact_Cshock(3,timei,gammai,machs,macha,xmin,xmax,xexact,yexact,ierr)
           elseif (iploty.eq.ivx+1 .and. ndimV > 1) then
-             call exact_Cshock(4,timei,gamma,machs,macha,xmin,xmax,xexact,yexact,ierr)
+             call exact_Cshock(4,timei,gammai,machs,macha,xmin,xmax,xexact,yexact,ierr)
           elseif (iploty.eq.iBfirst .and. iBfirst.gt.0) then
-             call exact_Cshock(5,timei,gamma,machs,macha,xmin,xmax,xexact,yexact,ierr)
+             call exact_Cshock(5,timei,gammai,machs,macha,xmin,xmax,xexact,yexact,ierr)
           endif
        endif
     end select
@@ -1146,8 +1228,8 @@ contains
     !----------------------------------------------------------
     !  plot this as a line on the current graph
     !----------------------------------------------------------
-    if (ierr <= 0) call plot_exact_solution(itransx,itransy,iexactpts,npart,&
-                                            xexact,yexact,xplot,yplot,imarker,iaxisy)
+    if (ierr <= 0) call plot_exact_solution(itransx,itransy,iexactpts,npart,xexact,yexact,xplot,yplot, &
+                                            itag,iamtype,noftype,iplot_type,xmin,xmax,imarker,iaxisy)
     !
     !--reset line and colour settings
     !
@@ -1168,20 +1250,29 @@ contains
   ! Wrapper routine to plot the exact solution line on current graph
   ! and calculate errors with respect to the data
   !------------------------------------------------------------------
-  subroutine plot_exact_solution(itransx,itransy,iexactpts,np,xexact,yexact,xplot,yplot,imarker,iaxisy,ls,err)
+  subroutine plot_exact_solution(itransx,itransy,iexactpts,np,xexact,yexact,xplot,yplot,&
+                                 itag,iamtype,noftype,iplot_type,xmin,xmax,imarker,iaxisy,ls,err)
    use transforms, only:transform,transform_inverse
-   use plotlib,    only:plot_line,plot_sls
+   use plotlib,    only:plot_line,plot_sls,plot_sci,plot_qci
+   use params,     only:int1,maxparttypes
    integer, intent(in)    :: itransx,itransy,iexactpts,np,imarker,iaxisy
    real,    intent(inout) :: xexact(:),yexact(:)
-   real,    intent(in)    :: xplot(:),yplot(:)
-   integer, intent(in), optional :: ls
-   logical, intent(in), optional :: err
+   real,    intent(in)    :: xplot(:),yplot(:),xmin,xmax
+   integer, intent(in)    :: itag(:)
+   integer(int1), intent(in) :: iamtype(:)
+   integer,       intent(in) :: noftype(maxparttypes)
+   logical,       intent(in) :: iplot_type(maxparttypes)
+   integer,       intent(in), optional :: ls
+   logical,       intent(in), optional :: err
    real :: residuals(np),ypart(np)
    real :: errL1,errL2,errLinf
-   integer :: iused,ierr
+   integer :: iused,ierr,iCurrentColour
    logical :: plot_err
    character(len=12) :: str1,str2
 
+   call plot_qci(iCurrentColour)
+   call plot_sci(iExactLineColour)
+
    if (itransx > 0) call transform(xexact(1:iexactpts),itransx)
    if (itransy > 0) call transform(yexact(1:iexactpts),itransy)
    
@@ -1203,33 +1294,45 @@ contains
       ypart(1:np) = yplot(1:np)
       if (itransy > 0) call transform_inverse(ypart(1:np),itransy)
       !--calculate errors
-      call calculate_errors(xexact(1:iexactpts),yexact(1:iexactpts), &
-                            xplot(1:np),ypart,residuals, &
+      call calculate_errors(xexact(1:iexactpts),yexact(1:iexactpts),xplot(1:np),ypart,&
+                            itag,iamtype,noftype,iplot_type,xmin,xmax,residuals, &
                             errL1,errL2,errLinf,iused)
       if (iused.ne.np) then
          write(str1,"(i12)",iostat=ierr) iused
          write(str2,"(i12)",iostat=ierr) np
-         print "(3(a,es10.3,1x),'(used ',a,'/',a,' parts)')",' L1 err = ',errL1,'L2 err = ',errL2, &
+         print "(3(a,es12.5,1x),'(used ',a,'/',a,' parts)')",' L1 err = ',errL1,'L2 err = ',errL2, &
                                'L(inf) err = ',errLinf,trim(adjustl(str1)),trim(adjustl(str2))
       else
-         print "(3(a,es10.3,1x))",' L1 err = ',errL1,'L2 err = ',errL2, &
+         print "(3(a,es12.5,1x))",' L1 err = ',errL1,'L2 err = ',errL2, &
                                'L(inf) err = ',errLinf
       endif
-      if (iPlotResiduals) call plot_residuals(xplot,residuals,imarker,iaxisy)
+      if (iPlotResiduals) then
+         call plot_sci(1)
+         call plot_residuals(xplot,residuals,imarker,iaxisy)
+      endif
    endif
+   call plot_sci(iCurrentColour)
 
   end subroutine plot_exact_solution
 
   !--------------------------------
   ! Calculate various error norms 
   !--------------------------------
-  subroutine calculate_errors(xexact,yexact,xpts,ypts,residual,errL1,errL2,errLinf,iused)
-   real, intent(in)  :: xexact(:),yexact(:),xpts(:),ypts(:)
-   real, intent(out) :: residual(size(xpts))
-   real, intent(out) :: errL1,errL2,errLinf
+  subroutine calculate_errors(xexact,yexact,xpts,ypts,itag,iamtype,noftype,iplot_type,&
+                              xmin,xmax,residual,errL1,errL2,errLinf,iused)
+   use part_utils, only:igettype
+   use params,     only:int1,maxparttypes
+   real,    intent(in)  :: xexact(:),yexact(:),xpts(:),ypts(:),xmin,xmax
+   integer, intent(in)  :: itag(:)
+   integer(int1), intent(in) :: iamtype(:)
+   integer,       intent(in) :: noftype(maxparttypes)
+   logical,       intent(in) :: iplot_type(maxparttypes)
+   real,    intent(out) :: residual(size(xpts))
+   real,    intent(out) :: errL1,errL2,errLinf
    integer, intent(out) :: iused
-   integer :: i,j,npart,nerr
+   integer :: i,j,npart,nerr,nused,itype
    real    :: xi,dy,dx,yexacti,err1,ymax
+   logical :: mixedtypes
 
    errL1 = 0.
    errL2 = 0.
@@ -1237,47 +1340,59 @@ contains
    residual = 0.
    npart = size(xpts)
    iused = 0
-   ymax = -huge(ymax)
+   ymax = 0.
    nerr = 0
+   nused = 0
+   mixedtypes = size(iamtype).gt.1
 
    do i=1,npart
       xi = xpts(i)
       yexacti = 0.
-      !
-      !--find nearest point in exact solution table
-      !
-      do j=1,size(xexact)-1
-         if (xexact(j).lt.xi .and. xexact(j+1).gt.xi) then
-            if (abs(residual(i)).gt.tiny(residual)) nerr = nerr + 1
-            !--linear interpolation from tabulated exact solution
-            dy = yexact(j+1) - yexact(j)
-            dx = xexact(j+1) - xexact(j)
-            if (dx.gt.0.) then
-               yexacti = yexact(j) + dy/dx*(xi - xexact(j))
-               residual(i) = ypts(i) - yexacti
-            elseif (dy.gt.0.) then
-               yexacti = yexact(j)
-               residual(i) = ypts(i) - yexacti
-            else
-               nerr = nerr + 1
-               residual(i) = 0.
-            endif
-            iused = iused + 1
-            ymax = max(ymax,abs(yexacti))
+      if (xi >= xmin .and. xi <= xmax .and. itag(i) > 0) then
+         if (mixedtypes) then
+            itype = min(max(int(iamtype(i)),1),maxparttypes)
+         else
+            itype = igettype(i,noftype)
          endif
-      enddo
-      err1 = abs(residual(i))
-      errL1 = errL1 + err1
-      errL2 = errL2 + err1**2
-      errLinf = max(errLinf,err1)
-      if (yexacti.gt.tiny(yexacti)) residual(i) = residual(i)/abs(yexacti)
+         if (iplot_type(itype)) then
+            !
+            !--find nearest point in exact solution table
+            !
+            do j=1,size(xexact)-1
+               if (xexact(j).lt.xi .and. xexact(j+1).gt.xi) then
+                  if (abs(residual(i)).gt.tiny(residual)) nerr = nerr + 1
+                  !--linear interpolation from tabulated exact solution
+                  dy = yexact(j+1) - yexact(j)
+                  dx = xexact(j+1) - xexact(j)
+                  if (dx.gt.0.) then
+                     yexacti = yexact(j) + dy/dx*(xi - xexact(j))
+                     residual(i) = ypts(i) - yexacti
+                  elseif (dy.gt.0.) then
+                     yexacti = yexact(j)
+                     residual(i) = ypts(i) - yexacti
+                  else
+                     nerr = nerr + 1
+                     residual(i) = 0.
+                  endif
+                  iused = iused + 1
+                  ymax = max(ymax,abs(yexacti))
+               endif
+            enddo
+            err1 = abs(residual(i))
+            errL1 = errL1 + err1
+            errL2 = errL2 + err1**2
+            errLinf = max(errLinf,err1)
+            if (yexacti.gt.tiny(yexacti)) residual(i) = residual(i)/abs(yexacti)
+            nused = nused + 1
+         endif
+      endif
    enddo
    !
    !--normalise errors (use maximum y value)
    !
-   if (ymax.gt.tiny(ymax)) then
-      errL1 = errL1/(npart*ymax)
-      errL2 = sqrt(errL2/(npart*ymax**2))
+   if (ymax.gt.tiny(ymax) .and. nused > 0) then
+      errL1 = errL1/(nused*ymax)
+      errL2 = sqrt(errL2/(nused*ymax**2))
       errLinf = errLinf/ymax
    else
       print "(a)",' error normalising errors'
