@@ -15,7 +15,7 @@
 !  a) You must cause the modified files to carry prominent notices
 !     stating that you changed the files and the date of any change.
 !
-!  Copyright (C) 2005-2013 Daniel Price. All rights reserved.
+!  Copyright (C) 2005-2014 Daniel Price. All rights reserved.
 !  Contact: daniel.price@monash.edu
 !
 !-----------------------------------------------------------------
@@ -395,9 +395,10 @@ end subroutine get_labels
 !
 !----------------------------------------------------------------
 subroutine check_labels
- use settings_data, only:ndim,ndimV,ncolumns,iverbose
- use labels,        only:ix,irho,ih,ipmass
- use particle_data, only:masstype
+ use settings_data,   only:ndim,ndimV,ncolumns,iverbose
+ use labels,          only:ix,irho,ih,ipmass
+ use particle_data,   only:masstype
+ use settings_render, only:icolour_particles
  implicit none
  integer :: i,ndimset
 
@@ -451,11 +452,10 @@ subroutine check_labels
                    '  until positions of density, smoothing length and particle', &
                    '  masses are known (specified using the integer variables ', &
                    '  irho,ih and ipmass in the read_data routine)'
+          icolour_particles = .true.
        elseif (irho.gt.0 .and. ih.gt.0 .and. ipmass.eq.0 .and. all(masstype(:,:).lt.tiny(0.))) then
           print "(2(/,a))",' WARNING: Particle masses not read as array but mass not set:', &
                            '          RENDERING WILL NOT WORK! '
-
-
        endif
     endif
  endif
@@ -469,8 +469,8 @@ end subroutine check_labels
 !----------------------------------------------------------------
 subroutine check_data_read
  use params,        only:maxplot,maxparttypes
- use settings_data, only:ncolumns,ndim,ndimV,ntypes
- use particle_data, only:npartoftype,iamtype
+ use settings_data, only:ncolumns,ndim,ndimV,ntypes,ivegotdata
+ use particle_data, only:npartoftype,iamtype,dat
  use labels,        only:labeltype
  implicit none
  integer :: i,j,ntoti,nunknown,itype
@@ -504,6 +504,12 @@ subroutine check_data_read
           endif
        enddo
     enddo
+    ntoti = sum(npartoftype(:,1))
+    if (ntoti > size(dat(:,1,1))) then
+       print "(2(a,i10),a)",' ERROR: size of dat array (',size(dat(:,1,1)),&
+             ') too small for number of particles (',ntoti,')'
+       ivegotdata = .false.
+    endif
     !if (debugmode) then
        !
        !--for mixed type storage, check that the number of particles
