@@ -505,6 +505,14 @@ subroutine set_labels
     labelvec(icol+1:icol+ndimV) = 'force'
     icol = icol + ndimV
     iBfirst = 0
+
+    iamvec(icol+1:icol+ndimV) = icol + 1
+    labelvec(icol+1:icol+ndimV) = 'del^{2} v'
+    icol = icol + ndimV
+
+    iamvec(icol+1:icol+ndimV) = icol + 1
+    labelvec(icol+1:icol+ndimV) = 'grad (div {\bf v})'
+    icol = icol + ndimV
  endif
  if (iformat.eq.5) then
     icol = icol + 1
