@@ -15,7 +15,7 @@
 !  a) You must cause the modified files to carry prominent notices
 !     stating that you changed the files and the date of any change.
 !
-!  Copyright (C) 2005-2014 Daniel Price. All rights reserved.
+!  Copyright (C) 2005-2015 Daniel Price. All rights reserved.
 !  Contact: daniel.price@monash.edu
 !
 !-----------------------------------------------------------------
@@ -71,14 +71,14 @@ module sphNGread
  integer :: nhydroarrays,nmhdarrays
  logical :: phantomdump,smalldump,mhddump,rtdump,usingvecp,igotmass,h2chem,rt_in_header
  logical :: usingeulr,cleaning
- logical :: batcode,tagged
+ logical :: batcode,tagged,debug
  integer, parameter :: maxarrsizes = 10
  integer, parameter :: maxinblock = 128 ! max allowed in each block
  integer, parameter :: lentag = 16
  character(len=lentag) :: tagarr(maxplot)
  integer, parameter :: itypemap_sink_phantom = 3
  integer, parameter :: itypemap_dust_phantom = 2
- integer, parameter :: itypemap_unknown_phantom = 7
+ integer, parameter :: itypemap_unknown_phantom = 8
 
  !------------------------------------------
  ! generic interface to utilities for tagged
@@ -119,7 +119,7 @@ contains
   select case(int(iphase))
   case(1:2)
     itypemap_phantom = iphase
-  case(3:5) ! put sinks as type 3, everything else shifted by one
+  case(3:6) ! put sinks as type 3, everything else shifted by one
     itypemap_phantom = iphase + 1
   case(-3) ! sink particles, either from external_binary or read from dump
     itypemap_phantom = itypemap_sink_phantom
@@ -408,6 +408,11 @@ contains
         endif
         if (phantomdump .and. nints < 7) ntypes = nints - 1
         if (iverbose.ge.1) print *,'npart = ',npart,' MPI blocks = ',nblocks
+        if (phantomdump) then
+           n1 = npartoftypei(1)
+        else
+           call extract('n1',n1,intarr,tags,nints,ierr)
+        endif
      else
         if (nints.lt.3) then
            if (.not.phantomdump) print "(a)",'WARNING: npart,n1,n2 NOT IN HEADER??'
@@ -732,6 +737,7 @@ contains
         endif
      endif
   else
+     npartoftype(:) = 0
      npartoftype(1) = npart
      npartoftype(2) = max(ntotal - npart,0)
   endif
@@ -918,6 +924,101 @@ contains
   endif
  end subroutine guess_labels
 
+ integer function assign_column(tag,iarr,ipos,ikind,imaxcolumnread) result(icolumn)
+  use labels, only:ih,irho,ix,ipmass
+  character(len=lentag), intent(in) :: tag
+  integer,               intent(in) :: iarr,ipos,ikind
+  integer,               intent(inout) :: imaxcolumnread
+
+  if (tagged .and. len_trim(tag) > 0) then
+     !
+     ! use the tags to put certain arrays in an assigned place
+     ! no matter what type is used for the variable in the file
+     ! and no matter what order they appear in the dump file
+     !
+     select case(trim(tag))
+     case('x')
+        icolumn = ix(1)
+     case('y')
+        icolumn = ix(2)
+     case('z')
+        icolumn = ix(3)
+     case('m')
+        icolumn = ipmass
+     case('h')
+        icolumn = ih
+     case('rho')
+        icolumn = irho
+     case('Bx')
+        icolumn = nhydroarrays + 1
+     case('By')
+        icolumn = nhydroarrays + 2
+     case('Bz')
+        icolumn = nhydroarrays + 3
+     case default
+        icolumn = max(nhydroarrays + nmhdarrays + 1,imaxcolumnread + 1)
+        if (iarr==1) then
+           if (ikind==4) then  ! real*4 array
+              istart_extra_real4 = min(istart_extra_real4,icolumn)
+              if (debug) print*,' istart_extra_real4 = ',istart_extra_real4
+           endif
+        endif
+     end select
+  else
+     !
+     ! this is old code handling the non-tagged format where
+     ! particular arrays are assumed to be in particular places
+     !
+     if (ikind==6) then   ! default reals
+        if (iarr.eq.1.and.((phantomdump.and.ipos.eq.4) &
+           .or.(.not.phantomdump.and.ipos.eq.6))) then
+           ! read x,y,z,m,h and then place arrays after always-present ones
+           ! (for phantom read x,y,z only)
+           icolumn = nhydroarrays+nmhdarrays + 1
+        elseif (.not.phantomdump .and. (iarr.eq.4 .and. ipos.le.3)) then
+           icolumn = nhydroarrays + ipos
+        else
+           icolumn = imaxcolumnread + 1
+        endif
+     elseif (ikind==4) then  ! real*4s
+        if (phantomdump) then
+           if (iarr.eq.1 .and. ipos.eq.1) then
+              icolumn = ih ! h is always first real4 in phantom dumps
+              !!--density depends on h being read
+              !required(ih) = .true.
+           elseif (iarr.eq.4 .and. ipos.le.3) then
+              icolumn = nhydroarrays + ipos
+           else
+              icolumn = max(nhydroarrays+nmhdarrays + 1,imaxcolumnread + 1)
+              if (iarr.eq.1) then
+                 istart_extra_real4 = min(istart_extra_real4,icolumn)
+                 if (debug) print*,' istart_extra_real4 = ',istart_extra_real4
+              endif
+           endif
+        else
+           if (iarr.eq.1 .and. ipos.eq.1) then
+              icolumn = irho ! density
+           elseif (iarr.eq.1 .and. smalldump .and. ipos.eq.2) then
+              icolumn = ih ! h which is real4 in small dumps
+           !--this was a bug for sphNG files...
+           !elseif (iarr.eq.4 .and. i.le.3) then
+           !   icolumn = nhydroarrays + ipos
+           else
+              icolumn = max(nhydroarrays+nmhdarrays + 1,imaxcolumnread + 1)
+              if (iarr.eq.1) then
+                 istart_extra_real4 = min(istart_extra_real4,icolumn)
+                 if (debug) print*,' istart_extra_real4 = ',istart_extra_real4
+              endif
+           endif
+        endif
+     else ! used for untagged format with real*8's
+        icolumn = imaxcolumnread + 1
+     endif
+  endif
+  imaxcolumnread = max(imaxcolumnread,icolumn)
+
+ end function assign_column
+
 end module sphNGread
 
 !----------------------------------------------------------------------
@@ -951,7 +1052,6 @@ subroutine read_data(rootname,indexstart,nstepsread)
   real,    dimension(maxparttypes) :: massoftypei
   real    :: pmassi,hi,rhoi,hrlim,rad2d
   logical :: iexist, doubleprec,imadepmasscolumn,gotbinary,gotiphase
-  logical :: debug
 
   character(len=len(rootname)+10) :: dumpfile
   character(len=100) :: fileident
@@ -968,7 +1068,7 @@ subroutine read_data(rootname,indexstart,nstepsread)
   real, dimension(maxinblock) :: dummyreal
   real :: hfact,omega
   logical :: skip_corrupted_block_3
-  character(len=lentag) :: tagsreal(maxinblock)
+  character(len=lentag) :: tagsreal(maxinblock), tagtmp
   
   integer, parameter :: splash_max_iversion = 1
 
@@ -1394,9 +1494,10 @@ subroutine read_data(rootname,indexstart,nstepsread)
                      print "(a)",'ERROR in memory allocation'
                      return
                   endif
+                  tagtmp = ''
                   do k=1,nreal(iarr)
-                     if (debug) print*,'DEBUG: reading sink array ',k,isize(iarr)
-                     if (tagged) read(iunit,end=33,iostat=ierr) !tagtemp
+                     if (tagged) read(iunit,end=33,iostat=ierr) tagtmp
+                     if (debug) print*,'DEBUG: reading sink array ',k,isize(iarr),' tag = ',trim(tagtmp)
                      read(iunit,end=33,iostat=ierr) dattemp(1:isize(iarr))
                      if (ierr /= 0) print*,' ERROR during read of sink particle data, array ',k
                      
@@ -1528,19 +1629,11 @@ subroutine read_data(rootname,indexstart,nstepsread)
          endif
 !        default reals may need converting
          do i=1,nreal(iarr)
-            if (iarr.eq.1.and.((phantomdump.and.i.eq.4) &
-               .or.(.not.phantomdump.and.i.eq.6))) then
-               ! read x,y,z,m,h and then place arrays after always-present ones
-               ! (for phantom read x,y,z only)
-               icolumn = nhydroarrays+nmhdarrays + 1
-            elseif (.not.phantomdump .and. (iarr.eq.4 .and. i.le.3)) then
-               icolumn = nhydroarrays + i
-            else
-               icolumn = imaxcolumnread + 1
-            endif
-            imaxcolumnread = max(imaxcolumnread,icolumn)
-            if (debug) print*,' reading real ',icolumn
-            if (tagged) read(iunit,end=33,iostat=ierr) tagarr(icolumn)
+            tagtmp = ''
+            if (tagged) read(iunit,end=33,iostat=ierr) tagtmp
+            icolumn = assign_column(tagtmp,iarr,i,6,imaxcolumnread)
+            if (tagged) tagarr(icolumn) = tagtmp
+            if (debug)  print*,' reading real ',icolumn,' tag = ',trim(tagtmp)
             if (required(icolumn)) then
                if (doubleprec) then
                   read(iunit,end=33,iostat=ierr) dattemp(1:isize(iarr))
@@ -1589,44 +1682,18 @@ subroutine read_data(rootname,indexstart,nstepsread)
 !        real4s may need converting
          imaxcolumnread = max(imaxcolumnread,icolumn)
          if ((nreal(iarr)+nreal4(iarr)).gt.6) imaxcolumnread = max(imaxcolumnread,6)
+
          do i=1,nreal4(iarr)
-            if (phantomdump) then
-               if (iarr.eq.1 .and. i.eq.1) then
-                  icolumn = ih ! h is always first real4 in phantom dumps
-                  !--density depends on h being read
-                  required(ih) = .true.
-                  !if (required(irho)) required(ih) = .true.
-                  !if (any(npartoftypei(2:).gt.0)) required(ih) = .true.
-               elseif (iarr.eq.4 .and. i.le.3) then
-                  icolumn = nhydroarrays + i
-               else
-                  icolumn = max(nhydroarrays+nmhdarrays + 1,imaxcolumnread + 1)
-                  if (iarr.eq.1) then
-                     istart_extra_real4 = min(istart_extra_real4,icolumn)
-                     if (debug) print*,' istart_extra_real4 = ',istart_extra_real4
-                  endif
-               endif
-            else
-               if (iarr.eq.1 .and. i.eq.1) then
-                  icolumn = irho ! density
-               elseif (iarr.eq.1 .and. smalldump .and. i.eq.2) then
-                  icolumn = ih ! h which is real4 in small dumps
-               !--this was a bug for sphNG files...
-               !elseif (iarr.eq.4 .and. i.le.3) then
-               !   icolumn = nhydroarrays + i
-               else
-                  icolumn = max(nhydroarrays+nmhdarrays + 1,imaxcolumnread + 1)
-                  if (iarr.eq.1) then
-                     istart_extra_real4 = min(istart_extra_real4,icolumn)
-                     if (debug) print*,' istart_extra_real4 = ',istart_extra_real4
-                  endif
-               endif
-            endif
-            imaxcolumnread = max(imaxcolumnread,icolumn)
-            if (debug) print*,'reading real4 ',icolumn
-            if (tagged) read(iunit,end=33,iostat=ierr) tagarr(icolumn)
+            tagtmp = ''
+            if (tagged) read(iunit,end=33,iostat=ierr) tagtmp
+            icolumn = assign_column(tagtmp,iarr,i,4,imaxcolumnread)
+            if (debug) print*,'reading real4 ',icolumn,' tag = ',trim(tagtmp)
+            if (tagged) tagarr(icolumn) = tagtmp
+
+            if (phantomdump .and. icolumn==ih) required(ih) = .true. ! h always required for density
+
             if (required(icolumn)) then
-               if (allocated(dattempsingle)) THEN
+               if (allocated(dattempsingle)) then
                   read(iunit,end=33,iostat=ierr) dattempsingle(1:isize(iarr))
                   dat(i1:i2,icolumn,j) = real(dattempsingle(1:isize(iarr)))
                else
@@ -1683,12 +1750,13 @@ subroutine read_data(rootname,indexstart,nstepsread)
                if (debug) print*,'debug: making density ',icolumn
             endif
          enddo
-         icolumn = imaxcolumnread
 !        real 8's need converting
          do i=1,nreal8(iarr)
-            icolumn = icolumn + 1
-            if (debug) print*,'reading real8 ',icolumn
-            if (tagged) read(iunit,end=33,iostat=ierr) tagarr(icolumn)
+            tagtmp = ''
+            if (tagged) read(iunit,end=33,iostat=ierr) tagtmp
+            icolumn = assign_column(tagtmp,iarr,i,8,imaxcolumnread)
+            if (debug) print*,'reading real8 ',icolumn,' tag = ',trim(tagtmp)
+            if (tagged) tagarr(icolumn) = tagtmp
             if (required(icolumn)) then
                read(iunit,end=33,iostat=ierr) dattemp(1:isize(iarr))
                dat(i1:i2,icolumn,j) = real(dattemp(1:isize(iarr)))
@@ -1731,7 +1799,8 @@ subroutine read_data(rootname,indexstart,nstepsread)
  !
  !--reset centre of mass to zero if environment variable "SSPLASH_RESET_CM" is set
  !
-    if (allocated(dat) .and. n1.GT.0 .and. lenvironment('SSPLASH_RESET_CM') .and. allocated(iphase)) then
+    if (allocated(dat) .and. n1.GT.0 .and. n1 <= size(dat(:,1,1)) &
+       .and. lenvironment('SSPLASH_RESET_CM') .and. allocated(iphase)) then
        call reset_centre_of_mass(dat(1:n1,1:3,j),dat(1:n1,4,j),iphase(1:n1),n1)
     endif
  ! 
@@ -1918,6 +1987,7 @@ subroutine read_data(rootname,indexstart,nstepsread)
 
      call set_labels
      if (.not.phantomdump) then
+        npartoftype(:,j) = 0
         npartoftype(1,j) = npart - nptmassi - nstar - nunknown
         npartoftype(2,j) = ntotal - npart
         npartoftype(3,j) = nptmassi
@@ -2026,7 +2096,7 @@ end subroutine read_data
 
 subroutine set_labels
   use labels, only:label,unitslabel,labelzintegration,labeltype,labelvec,iamvec, &
-              ix,ipmass,irho,ih,iutherm,ivx,iBfirst,idivB,iJfirst,icv,iradenergy
+              ix,ipmass,irho,ih,iutherm,ivx,iBfirst,idivB,iJfirst,icv,iradenergy,idustfrac
   use params
   use settings_data,   only:ndim,ndimV,ntypes,ncolumns,UseTypeInRenderings,debugmode
   use geometry,        only:labelcoord
@@ -2067,6 +2137,7 @@ subroutine set_labels
   !
   if (tagged) then
      do i=1,ncolumns
+        label(i) = tagarr(i)
         select case(trim(tagarr(i)))
         case('m')
            ipmass = i
@@ -2094,6 +2165,8 @@ subroutine set_labels
            iJfirst = i
         case('psi')
            label(i) = '\psi'
+        case('dustfrac')
+           idustfrac = i
         case('alpha')
            label(i) = '\alpha'
         case('alphaB')
@@ -2255,7 +2328,8 @@ subroutine set_labels
      labeltype(4) = 'ghost'
      labeltype(5) = 'star'
      labeltype(6) = 'dark matter'
-     labeltype(7) = 'unknown/dead'
+     labeltype(7) = 'bulge'
+     labeltype(8) = 'unknown/dead'
      UseTypeInRenderings(:) = .true.
      UseTypeInRenderings(3) = .false.
      if (lenvironment('SSPLASH_PLOT_DUST')) then
