@@ -48,6 +48,12 @@ program splash
 !
 !     -------------------------------------------------------------------------
 !     Version history/ Changelog:
+!     2.6.0  : (22/10/15)
+!             SILO, falcON and .pbob data reads implemented; bug fixes in gadget-hdf5 reader;
+!             can recognise particle types in ascii read; more robust sphNG read;
+!             dust fraction recognised in phantom data read; Toomre Q works in physical units;
+!             bug fix with disappearing units labels; bug fix in shock tube exact solution;
+!             added splash calc delta; splash to ascii keeps precision; better power spectra
 !     2.5.1  : (29/01/15)
 !             error bar style options; support for 5K displays; can plot vectors
 !             and render with colours if h not read; range restrictions apply during splash to grid;
@@ -364,7 +370,7 @@ program splash
   logical :: ihavereadfilenames,evsplash,doconvert,useall,iexist
   character(len=120) :: string
   character(len=12)  :: convertformat
-  character(len=*), parameter :: version = 'v2.5.1 [29th Jan. 2015]'
+  character(len=*), parameter :: version = 'v2.6.0 [22nd Oct. 2015]'
 
   !
   ! initialise some basic code variables
@@ -410,30 +416,30 @@ program splash
            i = i + 1
            call get_argument(i,string)
            read(string,*,iostat=ierr) ipickx
-           if (ierr /= 0) call print_usage(quit=.true.)
+           if (ierr /= 0 .or. ipickx <= 0) call print_usage(quit=.true.)
            nomenu = .true.
         case('y')
            i = i + 1
            call get_argument(i,string)
            read(string,*,iostat=ierr) ipicky
-           if (ierr /= 0) call print_usage(quit=.true.)
+           if (ierr /= 0 .or. ipicky <= 0) call print_usage(quit=.true.)
            nomenu = .true.
         case('render','r','ren')
            i = i + 1
            call get_argument(i,string)
            read(string,*,iostat=ierr) irender
-           if (ierr /= 0) call print_usage(quit=.true.)
+           if (ierr /= 0 .or. irender < 0) call print_usage(quit=.true.)
            nomenu = .true.
         case('contour','c','cont','con')
            i = i + 1
            call get_argument(i,string)
            read(string,*,iostat=ierr) icontour
-           if (ierr /= 0) call print_usage(quit=.true.)
+           if (ierr /= 0 .or. icontour < 0) call print_usage(quit=.true.)
         case('vec','vecplot')
            i = i + 1
            call get_argument(i,string)
            read(string,*,iostat=ierr) ivecplot
-           if (ierr /= 0) call print_usage(quit=.true.)
+           if (ierr /= 0 .or. ivecplot < 0) call print_usage(quit=.true.)
            nomenu = .true.
         case('dev','device')
            i = i + 1
